@@ -69,13 +69,13 @@ class CookiePolicy(http_cookiejar.DefaultCookiePolicy):
     Domain=localhost."""
 
     def return_ok_domain(self, cookie, request):
-        if cookie.domain == '.localhost':
+        if cookie.domain.endswith(request.origin_req_host):
             return True
         return http_cookiejar.DefaultCookiePolicy.return_ok_domain(
             self, cookie, request)
 
     def set_ok_domain(self, cookie, request):
-        if cookie.domain == '.localhost':
+        if cookie.domain.endswith(request.origin_req_host):
             return True
         return http_cookiejar.DefaultCookiePolicy.set_ok_domain(
             self, cookie, request)
@@ -196,6 +196,7 @@ class TestApp(object):
         if value is not None:
             invalid_value = (
                 "You should use a value like ('Basic', ('user', 'password'))"
+                " OR ('Bearer', 'token') OR ('JWT', 'token')"
             )
             if isinstance(value, (list, tuple)) and len(value) == 2:
                 authtype, val = value
@@ -204,6 +205,9 @@ class TestApp(object):
                     val = ':'.join(list(val))
                     val = b64encode(to_bytes(val)).strip()
                     val = val.decode('latin1')
+                elif authtype in ('Bearer', 'JWT') and val and \
+                        isinstance(val, (str, text_type)):
+                    val = val.strip()
                 else:
                     raise ValueError(invalid_value)
                 value = str('%s %s' % (authtype, val))
@@ -227,6 +231,9 @@ class TestApp(object):
         Sets a cookie to be passed through with requests.
 
         """
+        cookie_domain = self.extra_environ.get('HTTP_HOST', 'localhost')
+        cookie_domain = cookie_domain.split(':', 1)[0]
+        cookie_domain = '.' + cookie_domain
         value = escape_cookie_value(value)
         cookie = http_cookiejar.Cookie(
             version=0,
@@ -234,7 +241,7 @@ class TestApp(object):
             value=value,
             port=None,
             port_specified=False,
-            domain='.localhost',
+            domain=cookie_domain,
             domain_specified=True,
             domain_initial_dot=False,
             path='/',
@@ -499,6 +506,10 @@ class TestApp(object):
             if isinstance(value, forms.File):
                 if value.value:
                     _append_file([key] + list(value.value))
+                else:
+                    # If no file was uploaded simulate an empty file with no
+                    # name like real browsers do:
+                    _append_file([key, b'', b''])
             elif isinstance(value, forms.Upload):
                 file_info = [key, value.filename]
                 if value.content is not None:
@@ -602,7 +613,7 @@ class TestApp(object):
         # verify wsgi compatibility
         app = lint.middleware(self.app) if self.lint else self.app
 
-        ## FIXME: should it be an option to not catch exc_info?
+        # FIXME: should it be an option to not catch exc_info?
         res = req.get_response(app, catch_exc_info=True)
 
         # be sure to decode the content
@@ -614,7 +625,7 @@ class TestApp(object):
         res.app = app
         res.test_app = self
 
-        # We do this to make sure the app_iter is exausted:
+        # We do this to make sure the app_iter is exhausted:
         try:
             res.body
         except TypeError:  # pragma: no cover
@@ -664,7 +675,7 @@ class TestApp(object):
                 res)
         if status != res.status_int:
             raise AppError(
-                "Bad response: %s (not %s)", res_status, status)
+                "Bad response: %s (not %s)\n%s", res_status, status, res)
 
     def _check_errors(self, res):
         errors = res.errors
